// Interrupt handling TIMER 0 FOR REAL TIME CLOCK
// Compiler C18 V3.40, MPLAB V8.63, 18F2550 BOLT SYSTEM
// Connect the module BOLT WITH YOUR LCD and keypad.
// Allows the user to enter 2 HOURS OF ACTIVATE AND 2 HOURS
// TO DESACTIVATE relay. 
// The time is entered with a format of six consecutive digits
// HOURS-MINUTES-SECONDS ENDED WITH LETTER F. FOR EXAMPLE:
// ACT1> 110300 DES1> 122245  ACT2> 164400 DES2> 193359  
// The system then REQUESTS CURRENT TIME
// PROMT "SET>" For example: set> 072200 (ENTER "F")
// Once you enter the current time,
// the system displays on the left side of LCD
// STATE OF RELAY AND ON THE RIGHT SIDE, CURRENT TIME,
// For example: "REL ON 15:33:28" OR "REL OFF 18:23:45"

#include <p18cxxx.h>
#include <delays.h>
#include <timers.h>
#include <stdio.h>
#include <stdlib.h>
#include "18F2550BOLT.h"
#define CLOCK_TIMER0 12000000;	//es la frecuencia (hertz) con la que se modifica el registro de 16 bits del TIMER0

void timer_isr(void);
void LCD_CEROS(void);
void DISPLAY_LCD(void);
void SET_TIME(void);
void SET_ACT1(void);
void SET_ACT2(void);
void SET_DES1(void);
void SET_DES2(void);

extern void _startup( void ); // See c018i.c in your C18 compiler dir 
#pragma code _RESET_INTERRUPT_VECTOR = 0x000800 
void _reset( void ) 
{ 
    _asm goto _startup _endasm 
} 

#pragma code _HIGH_INTERRUPT_VECTOR = 0x000808
void _high_ISR (void)
{
	_asm goto timer_isr _endasm
}

#pragma code
int long Ticker; 
int Segundos=0; 
int segundo_previo=0;
int Year=0,Mes=0,Dia=0,Hora=0,Minuto=0; 		//HORA ACTUAL
int Hora_A1,Hora_A2,Hora_D1,Hora_D2,Min_A1,Min_A2,Min_D1,Min_D2,Seg_A1,Seg_A2,Seg_D1,Seg_D2;		//HORAS ACTIVACION Y DESACTIVACION
char CEROS[10]="00:00:00";	
char RELE1[10]="REL  ON";
char RELE2[10]="REL OFF";
#define BISIESTO(Year) (Year%4 == 0);

#pragma interrupt timer_isr

void timer_isr(void)
{
Ticker=Ticker-65536;                    // Decrementa ticker  
  if (Ticker<65536)                      // si ya transcurri un segundo 
  {
	Ticker=Ticker+CLOCK_TIMER0;      	//   Incrementa ticker 
    Segundos++;                           //   Incrementa numero de segundos 
  } 
				if(Segundos == 60) 
				{
				Minuto++; Segundos=0; 
    				if(Minuto == 60) 
					{
					Hora++; Minuto=0; 
      					if(Hora == 24) 
						{
						Dia++; Hora=0; 
  						} 
					}		
				}  
INTCONbits.TMR0IF = 0 ;				//habilita de nuevo interrupcin
}

void main() 
{
	init_bolt();
	PORTB=0;
	SET_ACT1();
	SET_DES1();
	SET_ACT2();
	SET_DES2();
	SET_TIME();
	Ticker=CLOCK_TIMER0;
	OpenTimer0 (TIMER_INT_ON & T0_SOURCE_INT & T0_16BIT);			//INICIALIZA PARAMETROS DEL TIMER 0
	INTCONbits.GIE = 1; 											//enable global interrupts
	LCD_CEROS();
  		while(1) 
  			{ 
			if (Segundos != segundo_previo) 
    			{
					GotoXY(0,0);
					if(Segundos==Seg_A1 & Minuto==Min_A1 & Hora==Hora_A1){RELE=1;PrintString(RELE1);}
					if(Segundos==Seg_A2 & Minuto==Min_A2 & Hora==Hora_A2){RELE=1;PrintString(RELE1);}
					if(Segundos==Seg_D1 & Minuto==Min_D1 & Hora==Hora_D1){RELE=0;PrintString(RELE2);}
					if(Segundos==Seg_D2 & Minuto==Min_D2 & Hora==Hora_D2){RELE=0;PrintString(RELE2);}
				segundo_previo=Segundos; 
      			DISPLAY_LCD();
    			}  
  			} 
} 

void LCD_CEROS(void)
{
 char HORA[]="TIME:";
 char CEROS[10]="00:00:00";		//HORA INICIAL
 InitLCD();
 ClearScreen();
 PrintString(HORA);
 GotoXY(1,0);
 PrintString(CEROS);
} 

void DISPLAY_LCD(void)
{
 char wob[10];
 GotoXY(1,0);
 sprintf(wob,"%02u",Hora);
 PrintString(wob); 
 GotoXY(1,3);
 sprintf(wob,"%02u",Minuto);
 PrintString(wob); 
 GotoXY(1,6);
 sprintf(wob,"%02u",Segundos);
 PrintString(wob); 
}

void SET_TIME(void)
{
char buf[3];
char SET[]="SET>";
InitLCD();
ClearScreen();
PrintString(SET);
GotoXY(1,0);
SetBot();				//INGRESA LA HORA (6 DIGITOS CONSECUTIVOS) Y "F" AL FINAL
GotoXY(1,0);
WrtBot();
buf[0]=bot[0];
buf[1]=bot[1];
buf[2]=0;
Hora=atoi(buf);
buf[0]=bot[2];
buf[1]=bot[3];
buf[2]=0;
Minuto=atoi(buf);
buf[0]=bot[4];
buf[1]=bot[5];
buf[2]=0;
Segundos=atoi(buf);
}

void SET_ACT1(void)
{
char wob[10];
char buf[3];
char SET[]="ACT1>";
InitLCD();
ClearScreen();
PrintString(SET);
GotoXY(1,0);
SetBot();				//INGRESA LA HORA (6 DIGITOS CONSECUTIVOS) Y "F" AL FINAL
GotoXY(1,0);
WrtBot();
buf[0]=bot[0];
buf[1]=bot[1];
buf[2]=0;
Hora_A1=atoi(buf);
buf[0]=bot[2];
buf[1]=bot[3];
buf[2]=0;
Min_A1=atoi(buf);
buf[0]=bot[4];
buf[1]=bot[5];
buf[2]=0;
Seg_A1=atoi(buf);
lcd_display1: 
 GotoXY(1,0);
 PrintString(CEROS);
 GotoXY(1,0);
 sprintf(wob,"%02u",Hora_A1);
 PrintString(wob); 
 GotoXY(1,3);
 sprintf(wob,"%02u",Min_A1);
 PrintString(wob); 
 GotoXY(1,6);
 sprintf(wob,"%02u",Seg_A1);
 PrintString(wob); 
 delay_ms(1000);
}

void SET_ACT2(void)
{
char wob[10];
char buf[3];
char SET[]="ACT2>";
InitLCD();
ClearScreen();
PrintString(SET);
GotoXY(1,0);
SetBot();				//INGRESA LA HORA (6 DIGITOS CONSECUTIVOS) Y "F" AL FINAL
GotoXY(1,0);
WrtBot();
buf[0]=bot[0];
buf[1]=bot[1];
buf[2]=0;
Hora_A2=atoi(buf);
buf[0]=bot[2];
buf[1]=bot[3];
buf[2]=0;
Min_A2=atoi(buf);
buf[0]=bot[4];
buf[1]=bot[5];
buf[2]=0;
Seg_A2=atoi(buf);
lcd_display2: 
 GotoXY(1,0);
 PrintString(CEROS);
 GotoXY(1,0);
 sprintf(wob,"%02u",Hora_A2);
 PrintString(wob); 
 GotoXY(1,3);
 sprintf(wob,"%02u",Min_A2);
 PrintString(wob); 
 GotoXY(1,6);
 sprintf(wob,"%02u",Seg_A2);
 PrintString(wob); 
 delay_ms(1000);
}

void SET_DES1(void)
{
char wob[10];
char buf[3];
char SET[]="DES1>";
InitLCD();
ClearScreen();
PrintString(SET);
GotoXY(1,0);
SetBot();				//INGRESA LA HORA (6 DIGITOS CONSECUTIVOS) Y "F" AL FINAL
GotoXY(1,0);
WrtBot();
buf[0]=bot[0];
buf[1]=bot[1];
buf[2]=0;
Hora_D1=atoi(buf);
buf[0]=bot[2];
buf[1]=bot[3];
buf[2]=0;
Min_D1=atoi(buf);
buf[0]=bot[4];
buf[1]=bot[5];
buf[2]=0;
Seg_D1=atoi(buf);
lcd_display3: 
 GotoXY(1,0);
 PrintString(CEROS);
 GotoXY(1,0);
 sprintf(wob,"%02u",Hora_D1);
 PrintString(wob); 
 GotoXY(1,3);
 sprintf(wob,"%02u",Min_D1);
 PrintString(wob); 
 GotoXY(1,6);
 sprintf(wob,"%02u",Seg_D1);
 PrintString(wob); 
 delay_ms(1000);
}

void SET_DES2(void)
{
char wob[10];
char buf[3];
char SET[]="DES2>";
InitLCD();
ClearScreen();
PrintString(SET);
GotoXY(1,0);
SetBot();				//INGRESA LA HORA (6 DIGITOS CONSECUTIVOS) Y "F" AL FINAL
GotoXY(1,0);
WrtBot();
buf[0]=bot[0];
buf[1]=bot[1];
buf[2]=0;
Hora_D2=atoi(buf);
buf[0]=bot[2];
buf[1]=bot[3];
buf[2]=0;
Min_D2=atoi(buf);
buf[0]=bot[4];
buf[1]=bot[5];
buf[2]=0;
Seg_D2=atoi(buf);
lcd_display4: 
 GotoXY(1,0);
 PrintString(CEROS);
 GotoXY(1,0);
 sprintf(wob,"%02u",Hora_D2);
 PrintString(wob); 
 GotoXY(1,3);
 sprintf(wob,"%02u",Min_D2);
 PrintString(wob); 
 GotoXY(1,6);
 sprintf(wob,"%02u",Seg_D2);
 PrintString(wob); 
 delay_ms(1000);
}
